body {
	font-family: Arial, Helvetica, sans-serif;
	font-size: small;
	background-color: #CCCCCC;
	color: #000000;
}

table {
	border-collapse: collapse;
}

table tr td {
	border: 1px solid #AD220B;
	background-color: #FFFFFF;
	padding: 2px;
	vertical-align: top;
}

table tr th {
	border: none;
	background-color: #333333;
	color: #FFFFFF;
	font-weight: bold;
	text-align: center;
	padding: 4px;
}
table tr th a {
	color: #FFFFFF;
	text-decoration: underline;
}
table tr th a:hover {
	color: #FFFFFF;
	text-decoration: none;
}

ul.menu {
	list-style: none;
	margin: 0;
	padding: 0;
}
ul.menu li a {
	float: left;
	display: block;
	border: 1px solid #333333;
	background-color: #FFFFCC;
	margin: 2px;
	margin-bottom: 20px;
	padding: 2px;
	text-decoration: none;
	color: #000000;
	font-weight: bold;
}
ul.menu li a:hover {
	background-color: #FFFF66;
}